package de.tu_darmstadt.tk.SmartHomeNetworkSim.control;

import java.util.Observer;


import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SimulationManager;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;

/**
 * Controller which allows interaction with the Model and Simulation
 *
 * @author Andreas T. Meyer-Berg
 */
public class Controller {
	/**
	 * {@link Model} which stores the smart home model
	 */
	private Model model;
	
	/**
	 * Controller for Import of user defined java classes
	 */
	private ImportController importController;
	
	/**
	 * Controller which allows configuration of the program
	 */
	private SettingsController settingsController;
	
	/**
	 * NetworkController for manipulation of the network model
	 */
	private NetworkController networkController;
	
	/**
	 * Create a new Controller, which controls the given model and allows
	 * manipulation of this model
	 * 
	 * @param model
	 *            model, that should be controlled
	 */
	public Controller(Model model) {
		this.model = model;
		settingsController = new SettingsController(this.model, this);
		importController = new ImportController(this.model, this);
		networkController = new NetworkController(this.model, this);
	}
	
	/**
	 * Returns the ImportController, which manages the user imported classes, 
	 * which can be used in the framework
	 * @return {@link ImportController}
	 */
	public ImportController getControllerImport(){
		return importController;
	}
	
	/**
	 * Returns the controller responsible for configuration of the program
	 * @return {@link SettingsController}
	 */
	public SettingsController getControllerConfiguration(){
		return settingsController;
	}
	
	/**
	 * Return the controller, managing the networkModel
	 * @return
	 */
	public NetworkController getNetworkController(){
		return networkController;
	}
	
	/**
	 * Notifies Observer, which may update their visualization/state
	 */
	public void notifyObservers(){
		model.setChanged();
		model.notifyObservers();
	}
	
	/**
	 * Adds Observer to the model, which will be notified, when the model changes
	 * 
	 * @param observer observer to be to notified on changes
	 */
	public void addObserver(Observer observer){
		model.addObserver(observer);
	}
	
	/**
	 * Removes Observer from the model. The Observer will no longer be notified on changes 
	 * 
	 * @param observer observer to be removed
	 */
	public void removeObserver(Observer observer){
		model.deleteObserver(observer);
	}
		
	// TODO: Maybe Settings controller ? 
	/**
	 * Reset Simulation 
	 * 
	 * @param timestep
	 */
	public void resetSimulation(long timestep){
		for(SmartDevice d: model.getDevices())
			for(Port p:d.getPorts())
				if(p.getLastTrigger()>timestep)
					p.setLastTrigger(timestep);
	}
	
	/**
	 * Returns the Simulation Manager of the Program
	 * @return Simulation Manager
	 */
	public SimulationManager getSimulationManager() {
		return model.getSim();
	}
}
